#include "http.h"

#include "WWWLib.h"
#include "WWWHTTP.h"
#include "WWWInit.h"


int printer (const char * fmt, va_list pArgs)
{
    return (vfprintf(stdout, fmt, pArgs));
}

int tracer (const char * fmt, va_list pArgs)
{
    return (vfprintf(stderr, fmt, pArgs));
}

void HTTPConnection::Kludge(void(*p)())
{
  m_Kludge = p;
}


void HTTPConnection::HTTPRequest::addBuffer(unsigned char * buf, int len)
{
  if(!m_Buffer) {
    m_Buffer = new unsigned char[len];
  } else {
    unsigned char * p  = new unsigned char[m_nSize + len];
    memcpy(p,m_Buffer,m_nSize);
    delete [] m_Buffer;
    m_Buffer = p;
  }
  memcpy(m_Buffer + m_nSize,buf,len);
  m_nSize += len;
}

HTTPConnection::HTTPConnection() 
{ 
  initMetaObject();

  pthread_mutex_init(&m_ListLock, 0);

  converters = HTList_new();		/* List of converters */
  encodings = HTList_new();		/* List of encoders */
  chunk = NULL;

  m_bThreadRunning = false;

  HTLibInit("ViewML", "0.10");
  
  HTPrint_setCallback(printer);
  HTTrace_setCallback(tracer);
  
#if 0
  HTSetTraceMessageMask("sop");
#endif
  
  /* Register the default set of transport protocols */
  HTTransportInit();
  
  /* Register the default set of protocol modules */
  HTProtocolInit();
  
  /* Register the default set of BEFORE and AFTER callback functions */
  HTNetInit();
  
  /* Register the default set of converters */
  HTConverterInit(converters);
  HTFormat_setConversion(converters);
  
  /* Register the default set of transfer encoders and decoders */
  HTTransferEncoderInit(encodings);
  HTFormat_setTransferCoding(encodings);
  
  /* Register the default set of MIME header parsers */
  HTMIMEInit();
  
}

HTTPConnection::~HTTPConnection()
{
  HTFormat_deleteAll();
  HTLibTerminate();  
  pthread_mutex_destroy(&m_ListLock);
}

int HTTPConnection::Connect(QObject * p)
{
  connect(p,SIGNAL(fileRequest(const char*)),SLOT(slotOpenURL(const char*)));
}

void HTTPConnection::slotOpenURL(const char * URL)
{
  AddReq(URL);
  StartThread();
}

void HTTPConnection::StartThread()
{
  if(m_bThreadRunning)
    return;
  
  m_bThreadRunning = true;
  pthread_create(&m_Thread,NULL,runthread,this);

  (*m_Kludge)();
}

const char * HTTPConnection::loadPage(const char * URL)
{
  char * c;
  request = HTRequest_new();
  
  HTRequest_setOutputFormat(request, WWW_SOURCE);
  HTRequest_setPreemptive(request, YES);
  if (URL) {
    char * cwd = HTGetCurrentDirectoryURL();
    char * absolute_url = HTParse(URL, cwd, PARSE_ALL);
    HTAnchor * anchor = HTAnchor_findAddress(absolute_url);
    chunk = HTLoadAnchorToChunk(anchor, request);
    HT_FREE(absolute_url);
    HT_FREE(cwd);
    /* If chunk != NULL then we have the data */
    if (chunk) {
      int sz = HTChunk_size(chunk);
      c = HTChunk_toCString(chunk);
    }
  } 
  
  HTRequest_delete(request);
  
  return c;
}


void * HTTPConnection::runthread(void * p)
{
  HTTPConnection * pThis = (HTTPConnection*)p;
  HTTPRequest * r = 0;

  r = pThis->GetReq();
  while(r) {
    pThis->getURL(r);
    pThis->AddFinished(r);
    r = pThis->GetReq();
  }

  pThis->m_bThreadRunning = false;

  return 0;
}

void HTTPConnection::StopThread()
{

}

int  HTTPConnection::getURL(HTTPRequest * r)
{
  const char * URL = r->getURL();
  request = HTRequest_new();
  
  HTRequest_setOutputFormat(request, WWW_SOURCE);
  HTRequest_setPreemptive(request, YES);
  if (URL) {
    char * cwd = HTGetCurrentDirectoryURL();
    char * absolute_url = HTParse(URL, cwd, PARSE_ALL);
    HTAnchor * anchor = HTAnchor_findAddress(absolute_url);
    chunk = HTLoadAnchorToChunk(anchor, request);
    HT_FREE(absolute_url);
    HT_FREE(cwd);
    /* If chunk != NULL then we have the data */
    if (chunk) {
      int sz = HTChunk_size(chunk);
      r->addBuffer((unsigned char*)HTChunk_data(chunk),sz);
    }
  } 
  
  HTRequest_delete(request);
  
  return 0;
}

void HTTPConnection::AddReq(const char * url)
{
  HTTPRequest * r = new HTTPRequest(url);
  pthread_mutex_lock(&m_ListLock);
  
  m_Reqs.push(r);
  pthread_mutex_unlock(&m_ListLock);
}

HTTPConnection::HTTPRequest * HTTPConnection::GetReq()
{
  HTTPRequest * r = 0;
  pthread_mutex_lock(&m_ListLock);
  if(m_Reqs.size()) {
    r = m_Reqs.front();

    m_Reqs.pop();

  }
  pthread_mutex_unlock(&m_ListLock);
  return r;
}

int HTTPConnection::ReqSize()
{
 pthread_mutex_lock(&m_ListLock);
 int r = m_Reqs.size();
 pthread_mutex_unlock(&m_ListLock);
 return r;

}

void HTTPConnection::AddFinished(HTTPRequest * r)
{
  pthread_mutex_lock(&m_ListLock);
  m_Finished.push(r);
  pthread_mutex_unlock(&m_ListLock);
}

HTTPConnection::HTTPRequest * HTTPConnection::GetFinished()
{
  HTTPRequest * r = 0;
  pthread_mutex_lock(&m_ListLock);
  if(m_Finished.size()) {
    r = m_Finished.front();
    m_Finished.pop();
  }
  pthread_mutex_unlock(&m_ListLock);
  return r;
}

int HTTPConnection::FinishedSize()
{
 pthread_mutex_lock(&m_ListLock);
 int r = m_Finished.size();
 pthread_mutex_unlock(&m_ListLock);
 return r;

}

#include "http.moc"


